@@ -7,13 +7,20 @@
     Usage: steamdeps dependencies.txt
 """
 
+import argparse
 import os
 import re
+import shlex
 import stat
 import subprocess
 import sys
 import tempfile
 
+try:
+    import typing
+except ImportError:
+    pass
+
 try:
     import apt
 except ImportError:
@@ -30,6 +37,64 @@ SUPPORTED_STEAM_DEPENDENCY_VERSION = ['1']
 _arch = None
 
 
+class OsRelease:
+    def __init__(self):
+        # type: () -> None
+        self._load_any()
+        self._is_like = self._data.get('ID_LIKE', '').split()
+
+    def _load_any(self):
+        # type: () -> None
+        for path in ('/etc/os-release', '/usr/lib/os-release'):
+            self._data = self._load('/etc/os-release')
+
+            if self._data:
+                return
+
+    def _load(
+        self,
+        path        # type: str
+    ):
+        # type: (...) -> typing.Dict[str, str]
+
+        data = {}       # type: typing.Dict[str, str]
+
+        try:
+            with open(path, 'r', encoding='utf-8', errors='replace') as reader:
+                for line in reader:
+                    try:
+                        line = line.strip()
+
+                        if not line or line.startswith('#'):
+                            continue
+
+                        key, value = line.split('=', 1)
+
+                        data[key] = ''.join(shlex.split(value))
+
+                    except Exception as e:
+                        print(
+                            'Warning: error parsing %s: line %r: %s'
+                            % (path, line, e),
+                            file=sys.stderr
+                        )
+                        continue
+        except OSError:
+            return {}
+        else:
+            return data
+
+    def is_os(self, os_id):
+        # type: (str) -> bool
+        if self._data.get('ID') == os_id:
+            return True
+
+        return os_id in self._is_like
+
+    def dump(self):
+        print(self._data, file=sys.stderr)
+
+
 #
 # Get the current package architecture
 # This may be different than the actual architecture for the case of i386
@@ -57,7 +122,7 @@ def get_full_package_name(name):
     else:
         return name
 
-#
+
 # N.B. Version checks are not supported on virtual packages
 #
 def is_provided(pkgname):
@@ -332,10 +397,10 @@ read line
 
 
 ###
-def check_config(config):
+def check_config(path, config):
     if "STEAM_RUNTIME" not in config:
         sys.stderr.write(
-            "Missing STEAM_RUNTIME definition in %s\n" % sys.argv[1])
+            "Missing STEAM_RUNTIME definition in %s\n" % path)
         return False
 
     if config["STEAM_RUNTIME"] not in SUPPORTED_STEAM_RUNTIME:
@@ -345,8 +410,7 @@ def check_config(config):
 
     if "STEAM_DEPENDENCY_VERSION" not in config:
         sys.stderr.write(
-            "Missing STEAM_DEPENDENCY_VERSION definition in %s\n" % sys.argv[
-                1])
+            "Missing STEAM_DEPENDENCY_VERSION definition in %s\n" % path)
         return False
 
     if config["STEAM_DEPENDENCY_VERSION"]\
@@ -369,15 +433,33 @@ def check_config(config):
 ###
 def main():
     config = {}
+    os_release = OsRelease()
 
-    # Check the command line arguments
-    if len(sys.argv) < 2:
-        sys.stderr.write("Usage: %s dependencies.txt\n" % sys.argv[0])
-        return 1
+    parser = argparse.ArgumentParser(description='Install Steam dependencies')
+    parser.add_argument(
+        '--dry-run',
+        action='store_true',
+        help="Don't install anything, just report",
+    )
+    parser.add_argument(
+        '--debug-dump-os-release',
+        action='store_true',
+        help=argparse.SUPPRESS,
+    )
+    parser.add_argument(
+        'dependencies',
+        metavar='$HOME/.steam/root/steamdeps.txt',
+        help='Path to steamdeps.txt',
+    )
+    args = parser.parse_args()
+
+    if args.debug_dump_os_release:
+        os_release.dump()
+        return 0
 
     # Make sure we can open the file
     try:
-        fp = open(sys.argv[1])
+        fp = open(args.dependencies)
     except Exception as e:
         sys.stderr.write("Couldn't open file: %s\n" % e)
         return 2
@@ -394,7 +476,7 @@ def main():
             config[match.group(1)] = match.group(2)
 
     # Check to make sure we have a valid config
-    if not check_config(config):
+    if not check_config(args.dependencies, config):
         return 3
 
     # Seek back to the beginning of the file
@@ -434,6 +516,23 @@ def main():
                 packages[package.name] = package
                 dependencies.append([package])
 
+    # The Steam container runtime (pressure-vessel) requires a setuid
+    # bubblewrap executable on some kernel configurations. Steam is
+    # unprivileged, so we have to get it from the host OS.
+    if (
+        # Debian's kernel doesn't allow unprivileged users to create
+        # new namespaces (https://bugs.debian.org/898446) so we need the
+        # setuid bubblewrap
+        os_release.is_os('debian')
+        # Ubuntu's kernel does allow that. We assume Ubuntu derivatives
+        # like Linux Mint will inherit that, rather than reverting to the
+        # Debian behaviour
+        and not os_release.is_os('ubuntu')
+    ):
+        package = Package('bubblewrap', [])
+        packages[package.name] = package
+        dependencies.append([package])
+
     # Print package dependency information for debug
     """
     for row in dependencies:
@@ -485,7 +584,10 @@ def main():
                 print("Package %s needs to be installed" % package.name,
                       file=sys.stderr)
 
-        return update_packages(needed)
+        if args.dry_run:
+            return 1
+        else:
+            return update_packages(needed)
     else:
         return 0
 
